// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Local modifications to this file are described in the README.chromium
// file.

/*
 * Readability. An Arc90 Lab Experiment.
 * Website: http://lab.arc90.com/experiments/readability
 * Source:  http://code.google.com/p/arc90labs-readability
 *
 * "Readability" is a trademark of Arc90 Inc and may not be used without explicit permission.
 *
 * Copyright (c) 2010 Arc90 Inc
 * Readability is licensed under the Apache License, Version 2.0.
**/
var t7_readability_js = {
    bdbconsole: (typeof console !== 'undefined' && console.log) ? console : {log: function() {}},
    dbg: function(s) {
        t7_readability_js.bdbconsole.log("Readability: " + s);
    },

    readStyle: "style-newspaper",
    readSize: "size-medium",
    readMargin: "margin-wide",

    document: document,
    clonedBody: null,
    originalHTML: '',
    distilledHTML: '',
    distilledArticleContent: null,
    nextPageLink: '',
    containerElement: null,
    buttonRect: null,

    version:                '1.7.1',
    iframeLoads:             0,
    convertLinksToFootnotes: false,
    reversePageScroll:       false, /* If they hold shift and hit space, scroll up */
    frameHack:               false, /**
                                      * The frame hack is to workaround a firefox bug where if you
                                      * pull content out of a frame and stick it into the parent element, the scrollbar won't appear.
                                      * So we fake a scrollbar in the wrapping div.
                                     **/
    biggestFrame:            false,
    flags:                   0x1 | 0x2 | 0x4,   /* Start with all flags set. */

    /* constants */
    FLAG_STRIP_UNLIKELYS:     0x1,
    FLAG_WEIGHT_CLASSES:      0x2,
    FLAG_CLEAN_CONDITIONALLY: 0x4,
    THRESHOLD_SUBJECT_LENGTH: 50,
    MAGNIFIER_BUTTON_WIDTH:   51,
    MAGNIFIER_BUTTON_HEIGHT:  51,
    SUBJECT_ID: "bdb-subject-button",
    IMG_BASE64:               "data:image/png;base64,"
        + "iVBORw0KGgoAAAANSUhEUgAAADMAAAAzCAYAAAA6oTAqAAAF5ElEQVR42u2YsYvqShTG73/24HWv"
        + "WR73wTYWW9hYbGFjkcLGwiJNCos0KYRnkyJFHrwUA4agYFgCKwsBF/EiKwguQiD/wfs+3+rOZhIT"
        + "1+WicIsPEufMzPnNOXNmzLfvTf0v6F8ouWL9Df3+jSAtw5l0+0Jcq1qG/UAgwiQdS4jrljcEx/IX"
        + "zKWp/aUwyFvLjR7c8HnmT1+WYbxeR8+vm+h5veEzf3NHcWy54eRiYQx7NBbRYj5dvCbxS5JWEWy3"
        + "HqCNgR98GYz5AcYTp0jv+0NC0LkzlLij51n3/5w/R5+HQao8cHXPAZHFVOwhwl8GgxdRRbaYPtGB"
        + "LxfSFIsUcY7PyZVgvhAkjF9SdzRNbS/cyRvFXP0qUJ8G0nonwHCSklRJTVukDc1IvzfauWp2zHTg"
        + "jmD/egToNTFsf3wmjCsKxOtCcKxaDbxRWmt2Jcc721pTX9VaxpJC2wq/Jft2Arv+tDiy2ENvqVNV"
        + "H2HwIoqEqrUsWMW0Y9p7iORO682auj0uGqfZHUzuWsaPPRQiWQjk+PGMfarLKYdByCdFE3Z6g/QG"
        + "TtFBXFSHcr8OqouOiFJ8/gClD8aM3M0OyCs6i1hhg6owLUOBqR4Vy/F3II22+ZjXD3voULr5nLOS"
        + "4k4zZogQU+686KgwjsiKK8qSmVetbu87ALEeaZenbB+eA3l2BKprBiNxsFUXwSnTRxi8iKz6bviY"
        + "t2JG32VqzWlTJBXGHRbZskD0UeVy5xr4E9qUyz4Og4vhj7xNX2vBHp1PgcHqBUW23EMo2/mVUkTx"
        + "J2BskZUfLTZ0XpYI4xRVK5btOFgbpVFWtl/HdMf7Ng3KzlVDEeFZle2HBV2xvUxNvQQGV/ckOzjT"
        + "ASsZyHbY7Uu2naK+Fz3JYzQ61oQ3BXXxnrcnw+BFZDVdKE7wbNhm7bhRT4This+z4zgiUuyQHYpv"
        + "qirBKGFnSd5k7UIk96kwthetFRg/UsbBSlWCudcHR2HopOLEwAsVGEQroW10kNoP0Tu0wZY3h1V2"
        + "HC+MFRikXjWYrgSDF5ERB+fV/IOYw1k73sVu6loqSenHc0luxw1gLo3BisZ0Vfph8VTfVJXDYLMn"
        + "2cGnEMugbAfHZrgNJLJyYA5tOPW3da0XyWN0LW9Cu6x6A29bDaZ/HKZrOls6n5Xlhk9lg2f7sAIe"
        + "s3f86TxvrpbeX1WBaXQkmAbu+VnhmrHMmYA5v8WGo02RcmGKbDXDCVBskpx5eNOIaVMqGQYvIiMe"
        + "ZI8oAwoMNfCiGW0KxPw/2PKZkxXYcm/mLhpv1EjHCW1KVA5Tb1vBfcfMhaEM7NmiwestPdX0/k58"
        + "xqGYB8P7X5w//hpRoV+WqAZjSTDspIqValkQHSohUF4/VKxEql5JPcfGckZPHKcoKrfN7ox2VVSX"
        + "YThZnu4QZqwQUuWFkxxLuUw/c4w0jSg+S20sw0N3JKeWKmREUm+bAe0rqS3B4EUUiNFZtY1+ZkK1"
        + "KHCludGLxkFJD3BuzLCHuNnLxkvaPWfMfuWqDsPoBEwV3XI4Uan86WKDkxvpGc0pRmE0fdmw7RQR"
        + "ujqQ+Q5z1zbFEfFgfAQQ/5hxop8mAmkAog9HpUkweBFlur3vxgTSjD7T4KcBhTsge1zi32kw1Pc3"
        + "IH4ns73wTEfXuIWLlAUGZ82pQOfDULdIuX3pbbR7/EopVbtS8eYMCJ8Q+9LNu1s1ICMfqKb13mHw"
        + "Ik5SywhucAbtneFnp1bX4p+33acjP3pmKlK7Z8ePdmdHS7f4AfAAgUo5x1hj9N9UAWKVw8fEQPVH"
        + "gsGL+IQQJWNCqH2kKirZQ0hjDW/q1YBYIRVfzoVRwfSITsKpNRzeQgnFZ6z8im20AcQwfwyjChDb"
        + "1qUwl6DbciB+VNmUwBjiQoQIS0AjAq0PGk0XLBzrnH7vMFgRcUEiUEAgFouuaTMauwO7BsAbpKti"
        + "L8Ogs7gwHYD+uNPSvbj/sO8C1fbyYaghCkcMqCUiMpNArhFG1S+Ya4X5B5pA4ooVQX3C/PkGlFyp"
        + "tgSBfvsPept2t9XARB8AAAAASUVORK5CYII=",

    maxPages:    30, /* The maximum number of pages to loop through before we call it quits and just show a link. */
    parsedPages: {}, /* The list of pages we've parsed in this call of readability, for autopaging. As a key store for easier searching. */
    pageETags:   {}, /* A list of the ETag headers of pages we've parsed, in case they happen to match, we'll know it's a duplicate. */

    /**
     * All of the regular expressions in use within readability.
     * Defined up here so we don't instantiate them repeatedly in loops.
     **/
    regexps: {
        unlikelyCandidates:    /combx|comment|community|disqus|extra|foot|header|menu|remark|rss|shoutbox|sidebar|sponsor|ad-break|agegate|pagination|pager|popup|tweet|twitter/i,
        okMaybeItsACandidate:  /and|article|body|column|main|shadow|news/i,
        ads:                   /sinaads/i,
        positive:              /article|body|content|entry|hentry|main|page|pagination|post|text|blog|story/i,
        negative:              /combx|comment|com-|contact|foot|footer|footnote|masthead|media|meta|outbrain|player|promo|related|scroll|shoutbox|sidebar|sponsor|shopping|tags|tool|widget/i,
        extraneous:            /print|archive|comment|discuss|e[\-]?mail|share|reply|all|login|sign|single/i,
        divToPElements:        /<(a|blockquote|dl|div|img|ol|p|pre|table|ul)/i,
        replaceBrs:            /(<br[^>]*>[ \n\r\t]*){2,}/gi,
        replaceFonts:          /<(\/?)font[^>]*>/gi,
        trim:                  /^\s+|\s+$/g,
        normalize:             /\s{2,}/g,
        killBreaks:            /(<br\s*\/?>(\s|&nbsp;?)*){1,}/g,
        videos:                /http:\/\/(www\.)?(youtube|vimeo)\.com/i,
        skipFootnoteLink:      /^\s*(\[?[a-z0-9]{1,2}\]?|^|edit|citation needed)\s*$/i,
        nextLink:              /(下一页|next|weiter|continue|>([^\|]|$)|»([^\|]|$))/i, // Match: next, continue, >, >>, » but not >|, »| as those usually mean last.
        prevLink:              /(上一页|prev|earl|old|new|<|«)/i
    },

    /**
     * Runs readability.
     *
     * Workflow:
     *  1. Prep the document by removing script tags, css, etc.
     *  2. Build readability's DOM tree.
     *  3. Grab the article content from the current dom tree.
     *  4. Replace the current DOM tree with the new one.
     *  5. Read peacefully.
     *
     * @return void
     **/
    init: function() {
        //t7_readability_js.originalHTML = document.documentElement.innerHTML;
        /* Before we do anything, remove all scripts that are not readability. */
        //window.onload = window.onunload = function() {};

        t7_readability_js.removeScripts(document);

        /* Make sure this document is added to the list of parsed pages first, so we don't double up on the first page */
        t7_readability_js.parsedPages[window.location.href.replace(/\/$/, '')] = true;

        /* Pull out any possible next page link first */
        t7_readability_js.nextPageLink = t7_readability_js.findNextPageLink(document.body);

        /* We handle processing of nextPage from C++ set nextPageLink to null */
        var nextPageLink = null;

        t7_readability_js.prepDocument();

        /* Build readability's DOM tree */
        var overlay        = document.createElement("DIV");
        var innerDiv       = document.createElement("DIV");
        var articleTools   = t7_readability_js.getArticleTools();
        var articleTitleText   = t7_readability_js.getArticleTitle();
        var articleContent = t7_readability_js.grabArticle();

        if(!articleContent) {
            articleContent    = document.createElement("DIV");
            articleContent.id = "readability-content";
            articleContent.innerHTML = [
                "<p>Sorry, readability was unable to parse this page for content. If you feel like it should have been able to, please <a href='http://code.google.com/p/arc90labs-readability/issues/entry'>let us know by submitting an issue.</a></p>",
                (t7_readability_js.frameHack ? "<p><strong>It appears this page uses frames.</strong> Unfortunately, browser security properties often cause Readability to fail on pages that include frames." : ""),
                "<p>Also, please note that Readability does not play very nicely with front pages. Readability is intended to work on articles with a sizable chunk of text that you'd like to read comfortably. If you're using Readability on a landing page (like nytimes.com for example), please click into an article first before using Readability.</p>"
            ].join('');

            nextPageLink = null;
            t7_readability_js.distilledHTML = "";
            t7_readability_js.removeIdForElements(null);
            return;
        }

        overlay.id              = "readOverlay";
        innerDiv.id             = "readInner";

        /* Apply user-selected styling */
        //document.body.className = t7_readability_js.readStyle;
        //document.dir            = t7_readability_js.getSuggestedDirection(articleTitleText);

        if (t7_readability_js.readStyle === "style-athelas" || t7_readability_js.readStyle === "style-apertura"){
            overlay.className = t7_readability_js.readStyle + " rdbTypekit";
        } else {
            overlay.className = t7_readability_js.readStyle;
        }
        innerDiv.className    = t7_readability_js.readMargin + " " + t7_readability_js.readSize;

        if(typeof(readConvertLinksToFootnotes) !== 'undefined' && readConvertLinksToFootnotes === true) {
            t7_readability_js.convertLinksToFootnotes = true;
        }

        if(t7_readability_js.frameHack) {
            var readOverlay = document.getElementById('readOverlay');
            readOverlay.style.height = '100%';
            readOverlay.style.overflow = 'auto';
        }

        /**
         * If someone tries to use Readability on a site's root page, give them a warning about usage.
        **/
        if((window.location.protocol + "//" + window.location.host + "/") === window.location.href) {
            articleContent.style.display = "none";
            var rootWarning = document.createElement('p');
                rootWarning.id = "readability-warning";
                rootWarning.innerHTML = "<em>Readability</em> was intended for use on individual articles and not home pages. " +
                "If you'd like to try rendering this page anyway, <a onClick='javascript:document.getElementById(\"readability-warning\").style.display=\"none\";document.getElementById(\"readability-content\").style.display=\"block\";'>click here</a> to continue.";

            //innerDiv.insertBefore( rootWarning, articleContent );
            t7_readability_js.distilledHTML = "";
            return;
        }

        t7_readability_js.postProcessContent(articleContent);

        t7_readability_js.distilledArticleContent = articleContent;
        // checkZoomLevel() will draw subject button if needed.
        t7_readability_js.checkZoomLevel();
        t7_readability_js.removeIdForElements(articleContent);
        t7_readability_js.distilledHTML = articleContent.innerHTML;
        document.bodyText = document.body.innerText;
        window.addEventListener('click', function() {
                if (document.body.innerText !== document.bodyText) {
                    document.bodyText = document.body.innerText;
                    t7_readability_js.onResize();
                }
            });
        //window.addEventListener('resize', t7_readability_js.onResize);
        //window.addEventListener('orientationchange', t7_readability_js.onResize);

        //window.scrollTo(0, 0);

        if (nextPageLink) {
            /**
             * Append any additional pages after a small timeout so that people
             * can start reading without having to wait for this to finish processing.
            **/
            window.setTimeout(function() {
                t7_readability_js.appendNextPage(nextPageLink);
            }, 500);
        }

        /** Smooth scrolling **/
        document.onkeydown = function(e) {
            var code = (window.event) ? event.keyCode : e.keyCode;
            if (code === 16) {
                t7_readability_js.reversePageScroll = true;
                return;
            }

            if (code === 32) {
                t7_readability_js.curScrollStep = 0;
                var windowHeight = window.innerHeight ? window.innerHeight : (document.documentElement.clientHeight ? document.documentElement.clientHeight : document.body.clientHeight);

                if(t7_readability_js.reversePageScroll) {
                    t7_readability_js.scrollTo(t7_readability_js.scrollTop(), t7_readability_js.scrollTop() - (windowHeight - 50), 20, 10);
                }
                else {
                    t7_readability_js.scrollTo(t7_readability_js.scrollTop(), t7_readability_js.scrollTop() + (windowHeight - 50), 20, 10);
                }

                return false;
            }
        };

        document.onkeyup = function(e) {
            var code = (window.event) ? event.keyCode : e.keyCode;
            if (code === 16) {
                t7_readability_js.reversePageScroll = false;
                return;
            }
        };
    },

    /**
     * Run any post-process modifications to article content as necessary.
     *
     * @param Element
     * @return void
    **/
    postProcessContent: function(articleContent) {
        if(t7_readability_js.convertLinksToFootnotes && !window.location.href.match(/wikipedia\.org/g)) {
            t7_readability_js.addFootnotes(articleContent);
        }

        t7_readability_js.fixImageFloats(articleContent);
    },

    /**
     * Some content ends up looking ugly if the image is too large to be floated.
     * If the image is wider than a threshold (currently 55%), no longer float it,
     * center it instead.
     *
     * @param Element
     * @return void
    **/
    fixImageFloats: function (articleContent) {
        var imageWidthThreshold = Math.min(articleContent.offsetWidth, 800) * 0.55,
            images              = articleContent.getElementsByTagName('img');

        for(var i=0, il = images.length; i < il; i+=1) {
            var image = images[i];

            if(image.offsetWidth > imageWidthThreshold) {
                image.className += " blockImage";
            }
        }
    },

    /**
     * Get the article tools Element that has buttons like reload, print.
     *
     * @return void
     **/
    getArticleTools: function () {
        var articleTools = document.createElement("DIV");

        articleTools.id        = "readTools";
        articleTools.innerHTML =
            "<a href='#' onclick='return window.location.reload()' title='Reload original page' id='reload-page'>Reload Original Page</a>" +
            "<a href='#' onclick='javascript:window.print();' title='Print page' id='print-page'>Print Page</a>" +
            "<a href='#' onclick='t7_readability_js.emailBox(); return false;' title='Email page' id='email-page'>Email Page</a>";

        return articleTools;
    },

    /**
     * retuns the suggested direction of the string
     *
     * @return "rtl" || "ltr"
     **/
    getSuggestedDirection: function(text) {
        function sanitizeText() {
            return text.replace(/@\w+/, "");
        }

        function countMatches(match) {
            var matches = text.match(new RegExp(match, "g"));
            return matches !== null ? matches.length : 0;
        }

        function isRTL() {
            var count_heb =  countMatches("[\\u05B0-\\u05F4\\uFB1D-\\uFBF4]");
            var count_arb =  countMatches("[\\u060C-\\u06FE\\uFB50-\\uFEFC]");

            // if 20% of chars are Hebrew or Arbic then direction is rtl
            return  (count_heb + count_arb) * 100 / text.length > 20;
        }

        text  = sanitizeText(text);
        return isRTL() ? "rtl" : "ltr";
    },

    /**
     * Get the article title as an H1.
     *
     * @return void
     **/
    getArticleTitle: function () {
        var curTitle = "",
            origTitle = "";

        try {
            curTitle = origTitle = document.title;
            if(typeof curTitle !== "string") { /* If they had an element with id "title" in their HTML */
                curTitle = origTitle = t7_readability_js.getInnerText(document.getElementsByTagName('title')[0]);
            }
        }
        catch(e) {}

        if(curTitle.match(/ [\|\-] /))
        {
            curTitle = origTitle.replace(/(.*)[\|\-] .*/gi,'$1');

            if(curTitle.split(' ').length < 3) {
                curTitle = origTitle.replace(/[^\|\-]*[\|\-](.*)/gi,'$1');
            }
        }
        else if(curTitle.indexOf(': ') !== -1)
        {
            curTitle = origTitle.replace(/.*:(.*)/gi, '$1');

            if(curTitle.split(' ').length < 3) {
                curTitle = origTitle.replace(/[^:]*[:](.*)/gi,'$1');
            }
        }
        else if(curTitle.length > 150 || curTitle.length < 15)
        {
            var hOnes = document.getElementsByTagName('h1');
            if(hOnes.length === 1)
            {
                curTitle = t7_readability_js.getInnerText(hOnes[0]);
            }
        }

        curTitle = curTitle.replace( t7_readability_js.regexps.trim, "" );

        if(curTitle.split(' ').length <= 4) {
            curTitle = origTitle;
        }
        return curTitle;
    },

    /**
     * Prepare the HTML document for readability to scrape it.
     * This includes things like stripping javascript, CSS, and handling terrible markup.
     *
     * @return void
     **/
    prepDocument: function () {
        /**
         * In some cases a body element can't be found (if the HTML is totally hosed for example)
         * so we create a new body node and append it to the document.
         */
        if(document.body === null)
        {
            var body = document.createElement("body");
            try {
                document.body = body;
            }
            catch(e) {
                document.documentElement.appendChild(body);
                t7_readability_js.dbg(e);
            }
            t7_readability_js.clonedBody = body.cloneNode(true);
        }

        var frames = document.getElementsByTagName('frame');
        if(frames.length > 0)
        {
            var bestFrame = null;
            var bestFrameSize = 0;    /* The frame to try to run readability upon. Must be on same domain. */
            var biggestFrameSize = 0; /* Used for the error message. Can be on any domain. */
            for(var frameIndex = 0; frameIndex < frames.length; frameIndex+=1)
            {
                var frameSize = frames[frameIndex].offsetWidth + frames[frameIndex].offsetHeight;
                var canAccessFrame = false;
                try {
                    var frameBody = frames[frameIndex].contentWindow.document.body;
                    canAccessFrame = true;
                }
                catch(eFrames) {
                    t7_readability_js.dbg(eFrames);
                }

                if(frameSize > biggestFrameSize) {
                    biggestFrameSize         = frameSize;
                    t7_readability_js.biggestFrame = frames[frameIndex];
                }

                if(canAccessFrame && frameSize > bestFrameSize)
                {
                    t7_readability_js.frameHack = true;

                    bestFrame = frames[frameIndex];
                    bestFrameSize = frameSize;
                }
            }

            if(bestFrame)
            {
                t7_readability_js.document = bestFrame.contentWindow.document;
            }
        }

        t7_readability_js.assignIdForElements();
        t7_readability_js.clonedBody = document.body.cloneNode(true);

        /* Remove all stylesheets */

        /* Remove all style tags in head (not doing this on IE) - TODO: Why not? */

        /* Turn all double br's into p's */
        /* Note, this is pretty costly as far as processing goes. Maybe optimize later. */
        t7_readability_js.replaceDoubleBrsWithPs(t7_readability_js.clonedBody);
        t7_readability_js.replaceFontsWithSpans(t7_readability_js.clonedBody);
    },


    /**
     * Prepare the article node for display. Clean out any inline styles,
     * iframes, forms, strip extraneous <p> tags, etc.
     *
     * @param Element
     * @return void
     **/
    prepArticle: function (articleContent) {
        t7_readability_js.cleanStyles(articleContent);
        t7_readability_js.killBreaks(articleContent);

        /* Clean out junk from the article content */
        t7_readability_js.cleanConditionally(articleContent, "form");
        t7_readability_js.clean(articleContent, "object");
        var h1s = articleContent.getElementsByTagName('h1');
        if (h1s.length === 1 && h1s[0].innerText === t7_readability_js.getArticleTitle())
            t7_readability_js.clean(articleContent, "h1");

        /**
         * If there is only one h2, they are probably using it
         * as a header and not a subheader, so remove it since we already have a header.
        ***/
        if(articleContent.getElementsByTagName('h2').length === 1) {
            t7_readability_js.clean(articleContent, "h2");
        }
        t7_readability_js.clean(articleContent, "iframe");

        t7_readability_js.cleanHeaders(articleContent);

        /* Do these last as the previous stuff may have removed junk that will affect these */
        t7_readability_js.cleanConditionally(articleContent, "table");
        t7_readability_js.cleanConditionally(articleContent, "ul");
        t7_readability_js.cleanConditionally(articleContent, "div");
        t7_readability_js.cleanConditionally(articleContent, "ins");

        /* Remove extra paragraphs */
        var articleParagraphs = articleContent.getElementsByTagName('p');
        for(var i = articleParagraphs.length-1; i >= 0; i-=1) {
            var imgCount    = articleParagraphs[i].getElementsByTagName('img').length;
            var embedCount  = articleParagraphs[i].getElementsByTagName('embed').length;
            var objectCount = articleParagraphs[i].getElementsByTagName('object').length;

            if(imgCount === 0 && embedCount === 0 && objectCount === 0 && t7_readability_js.getInnerText(articleParagraphs[i], false) === '') {
                articleParagraphs[i].parentNode.removeChild(articleParagraphs[i]);
            }
        }

        try {
            t7_readability_js.replaceBrsWithPs(articleContent);
        }
        catch (e) {
            t7_readability_js.dbg("Cleaning innerHTML of breaks failed. This is an IE strict-block-elements bug. Ignoring.: " + e);
        }
    },

    /**
     * Initialize a node with the readability object. Also checks the
     * className/id for special names to add to its score.
     *
     * @param Element
     * @return void
    **/
    initializeNode: function (node) {
        node.readability = {"contentScore": 0};

        switch(node.tagName) {
            case 'DIV':
            case 'ARTICLE':
                node.readability.contentScore += 5;
                break;

            case 'PRE':
            case 'TD':
            case 'BLOCKQUOTE':
                node.readability.contentScore += 3;
                break;

            case 'ADDRESS':
            case 'OL':
            case 'UL':
            case 'DL':
            case 'DD':
            case 'DT':
            case 'LI':
            case 'FORM':
                node.readability.contentScore -= 3;
                break;

            case 'H1':
            case 'H2':
            case 'H3':
            case 'H4':
            case 'H5':
            case 'H6':
            case 'TH':
                node.readability.contentScore -= 5;
                break;
        }

        node.readability.contentScore += t7_readability_js.getClassWeight(node);
    },

    nodeInfo: function(node, dumpHTML) {
        if (!node)
            return "" + node;

        var info = node
                + "("
                + node.tagName
                + ":" + node.className
                + ":" + node.id
                + ")";
        if (dumpHTML)
            info = info + ":" + node.outerHTML;
        return info;
    },

    /***
     * grabArticle - Using a variety of metrics (content score, classname, element types), find the content that is
     *               most likely to be the stuff a user wants to read. Then return it wrapped up in a div.
     *
     * @param page a document to run upon. Needs to be a full document, complete with body.
     * @return Element
    **/
    grabArticle: function (page, isPaging) {
        var stripUnlikelyCandidates = t7_readability_js.flagIsActive(t7_readability_js.FLAG_STRIP_UNLIKELYS);

        // Never work on the actual page.
        page = page ? page.cloneNode(true) : t7_readability_js.clonedBody;

        var allElements = page.getElementsByTagName('*');

        /**
         * First, node prepping. Trash nodes that look cruddy (like ones with the class name "comment", etc), and turn divs
         * into P tags where they have been used inappropriately (as in, where they contain no other block level elements.)
         *
         * Note: Assignment from index for performance. See http://www.peachpit.com/articles/article.aspx?p=31567&seqNum=5
         * TODO: Shouldn't this be a reverse traversal?
        **/
        var node = null;
        var nodesToScore = [];
        for(var nodeIndex = 0; (node = allElements[nodeIndex]); nodeIndex+=1) {
            if (!t7_readability_js.nodeIsVisible(node)) {
                if (node.parentNode && node.tagName !== "BR") {
                    //t7_readability_js.dbg("Skipping non-visible node: " + t7_readability_js.nodeInfo(node));
                    node.parentNode.removeChild(node);
                    nodeIndex -= 1;
                }
                continue;
            }
            /* Remove unlikely candidates */
            if (stripUnlikelyCandidates) {
                var unlikelyMatchString = node.className + '-' + node.id;
                if (
                    (
                        unlikelyMatchString.search(t7_readability_js.regexps.unlikelyCandidates) !== -1 &&
                        unlikelyMatchString.search(t7_readability_js.regexps.okMaybeItsACandidate) === -1 &&
                        node.tagName !== "BODY"
                    )
                )
                {
                    t7_readability_js.dbg("Removing unlikely candidate - " + unlikelyMatchString);
                    node.parentNode.removeChild(node);
                    nodeIndex-=1;
                    continue;
                }
            }

            if (node.tagName === "P" || node.tagName === "TD" || node.tagName === "PRE") {
                nodesToScore[nodesToScore.length] = node;
            }

            /* Turn all divs that don't have children block level elements into p's */
            if (node.tagName === "DIV") {
                if (node.innerHTML.search(t7_readability_js.regexps.divToPElements) === -1) {
                    var newNode = document.createElement('p');
                    newNode.visible = true;
                    try {
                        //newNode.originalNode = t7_readability_js.getOriginalNode(node);
                        t7_readability_js.moveNodeInnards(node, newNode);
                        node.parentNode.replaceChild(newNode, node);

                        nodesToScore[nodesToScore.length] = newNode;
                    }
                    catch(e) {
                        t7_readability_js.dbg("Could not alter div to p, probably an IE restriction, reverting back to div.: " + e);
                    }
                }
                else
                {
                    /* EXPERIMENTAL */
                    for(var i = 0, il = node.childNodes.length; i < il; i+=1) {
                        var childNode = node.childNodes[i];
                        if(childNode.nodeType === 3) { // Node.TEXT_NODE
                            var p = document.createElement('p');
                            var t = document.createTextNode(childNode.nodeValue);
                            p.appendChild(t);
                            p.style.display = 'inline';
                            p.className = 'readability-styled';
                            // We have skipped non-visible nodes in the beginning of the top for loop
                            p.visible = true;
                            childNode.parentNode.replaceChild(p, childNode);
                        }
                    }
                }
            }
        }

        /**
         * Loop through all paragraphs, and assign a score to them based on how content-y they look.
         * Then add their score to their parent node.
         *
         * A score is determined by things like number of commas, class names, etc. Maybe eventually link density.
        **/
        var candidates = [];
        for (var pt=0; pt < nodesToScore.length; pt+=1) {
            var parentNode      = nodesToScore[pt].parentNode;
            var grandParentNode = parentNode ? parentNode.parentNode : null;
            var innerText       = t7_readability_js.getInnerText(nodesToScore[pt]);

            if(!parentNode || typeof(parentNode.tagName) === 'undefined') {
                continue;
            }

            /* If this paragraph is less than 25 characters, don't even count it. */
            if(innerText.length < 25
                    && nodesToScore[pt].getElementsByTagName("img").length === 0) {
                if (candidates.length == 0) {
                    parentNode.removeChild(nodesToScore[pt]);
                    pt-=1;
                }
                continue;
            }

            /* Initialize readability data for the parent. */
            if(typeof parentNode.readability === 'undefined') {
                t7_readability_js.initializeNode(parentNode);
                candidates.push(parentNode);
            }

            /* Initialize readability data for the grandparent. */
            if(grandParentNode && typeof(grandParentNode.readability) === 'undefined' && typeof(grandParentNode.tagName) !== 'undefined') {
                t7_readability_js.initializeNode(grandParentNode);
                candidates.push(grandParentNode);
            }

            var contentScore = 0;

            /* Add a point for the paragraph itself as a base. */
            contentScore+=1;

            /* Add points for any commas within this paragraph */
            contentScore += innerText.split(',').length;
            contentScore += innerText.split('，').length; /* commas for Chinese */

            /* For every 100 characters in this paragraph, add another point. Up to 3 points. */
            contentScore += Math.min(Math.floor(innerText.length / 100), 3);

            /* Add the score to the parent. The grandparent gets half. */
            parentNode.readability.contentScore += contentScore;

            if(grandParentNode) {
                grandParentNode.readability.contentScore += contentScore/2;
            }
        }

        /**
         * After we've calculated scores, loop through all of the possible candidate nodes we found
         * and find the one with the highest score.
        **/
        var topCandidate = null, indexOfTopCandidate = -1;
        for(var c=0, cl=candidates.length; c < cl; c+=1)
        {
            /**
             * Scale the final candidates score based on link density. Good content should have a
             * relatively small link density (5% or less) and be mostly unaffected by this operation.
            **/
            candidates[c].readability.contentScore = candidates[c].readability.contentScore * (1-t7_readability_js.getLinkDensity(candidates[c]));

            t7_readability_js.dbg('Candidate: ' + t7_readability_js.nodeInfo(candidates[c]) + " with score " + candidates[c].readability.contentScore);

            if(indexOfTopCandidate < 0 || candidates[c].readability.contentScore > candidates[indexOfTopCandidate].readability.contentScore) {
                indexOfTopCandidate = c; }
        }

        /**
         * If we still have no top candidate, just use the body as a last resort.
         * We also have to copy the body node so it is something we can modify.
         **/
        if (indexOfTopCandidate < 0 || candidates[indexOfTopCandidate].tagName === "BODY")
        {
            topCandidate = document.createElement("DIV");
            t7_readability_js.moveNodeInnards(page, topCandidate);
            page.appendChild(topCandidate);
            t7_readability_js.initializeNode(topCandidate);
            candidates[indexOfTopCandidate] = topCandidate;
            topCandidate.visible = t7_readability_js.nodeIsVisible(document.body, false);
            t7_readability_js.dbg("topCandidate adjusted,  index=" + indexOfTopCandidate);
        } else {
            topCandidate = candidates[indexOfTopCandidate];
        }

        /**
         * Now that we have the top candidate, look through its siblings for content that might also be related.
         * Things like preambles, content split by ads that we removed, etc.
        **/
        var articleContent        = document.createElement("DIV");
        if (isPaging) {
            articleContent.id     = "readability-content";
        }
        var siblingScoreThreshold = Math.max(10, topCandidate.readability.contentScore * 0.2);
        var siblingNodes          =  [];
        /* Get count parts(similar score, same depth) and sibling nodes of top candidate */
        var getNodeDepth = function(node) {
            var d = 0;
            while(node && node.parentNode) {
                node = node.parentNode;
                d+=1;
            }
            return d;
        };
        var dTop = getNodeDepth(topCandidate);
        for(var c=0, cl=candidates.length; c < cl; c+=1) {
            if (candidates[c] == topCandidate
                || (candidates[c].readability.contentScore > topCandidate.readability.contentScore * 0.5
                    && getNodeDepth(candidates[c]) == dTop)) {
                var siblings = candidates[c].parentNode.childNodes;
                for (var s = 0, sl = siblings.length; s < sl; s+=1) {
                    if (siblings[s].nodeType === Node.ELEMENT_NODE || siblings[s].nodeType === Node.TEXT_NODE)
                        siblingNodes.push(siblings[s]);
                    else {
                        //t7_readability_js.dbg('Skipping unexpected nodeType: ' + siblings[s].nodeType);
                    }
                }
            }
        }

        for(var s=0, sl=siblingNodes.length; s < sl; s+=1) {
            var siblingNode = siblingNodes[s];
            var append      = false;

            /**
             * Fix for odd IE7 Crash where siblingNode does not exist even though this should be a live nodeList.
             * Example of error visible here: http://www.esquire.com/features/honesty0707
            **/
            if(!siblingNode || !t7_readability_js.nodeIsVisible(siblingNode)) {
                t7_readability_js.dbg("Skipping non-visible sibling node: " + t7_readability_js.nodeInfo(siblingNode));
                continue;
            }

            //t7_readability_js.dbg("Looking at sibling node: " + t7_readability_js.nodeInfo(siblingNode) + ((typeof siblingNode.readability !== 'undefined') ? (" with score " + siblingNode.readability.contentScore) : ''));

            if(siblingNode === topCandidate)
            {
                append = true;
            }

            var contentBonus = 0;
            /* Give a bonus if sibling nodes and top candidates have the example same classname */
            if(siblingNode.className === topCandidate.className && topCandidate.className !== "") {
                if (typeof siblingNode.readability === 'undefined') {
                    t7_readability_js.initializeNode(siblingNode);
                }
                contentBonus += topCandidate.readability.contentScore * 0.2;
            }

            if(typeof siblingNode.readability !== 'undefined' && (siblingNode.readability.contentScore+contentBonus) >= siblingScoreThreshold)
            {
                append = true;
            }

            if(siblingNode.nodeName === "P") {
                var linkDensity = t7_readability_js.getLinkDensity(siblingNode);
                var nodeContent = t7_readability_js.getInnerText(siblingNode);
                var nodeLength  = nodeContent.length;

                if(nodeLength > 80 && linkDensity < 0.25)
                {
                    append = true;
                }
                else if(nodeLength < 80 && linkDensity === 0 && nodeContent.search(/\.( |$)/) !== -1)
                {
                    append = true;
                }
            }

            if(append) {
                t7_readability_js.dbg("Appending node: " + t7_readability_js.nodeInfo(siblingNode));

                var nodeToAppend = null;
                if(siblingNode.nodeName !== "DIV" && siblingNode.nodeName !== "P") {
                    /* We have a node that isn't a common block level element, like a form or td tag. Turn it into a div so it doesn't get filtered out later by accident. */

                    t7_readability_js.dbg("Altering siblingNode of " + siblingNode.nodeName + ' to div.');
                    nodeToAppend = document.createElement("DIV");
                    try {
                        nodeToAppend.id = siblingNode.id;
                        nodeToAppend.readability = siblingNode.readability;
                        t7_readability_js.moveNodeInnards(siblingNode, nodeToAppend);
                    }
                    catch(er) {
                        t7_readability_js.dbg("Could not alter siblingNode to div, probably an IE restriction, reverting back to original.");
                        nodeToAppend = siblingNode;
                    }
                } else {
                    nodeToAppend = siblingNode;
                }

                /* To ensure a node does not interfere with readability styles, remove its classnames */
                nodeToAppend.className = "";

                /* Append sibling and subtract from our list because it removes the node when you append to another node */
                articleContent.appendChild(nodeToAppend);
            }
        }

        /**
         * So we have all of the content that we need. Now we clean it up for presentation.
        **/
        t7_readability_js.prepArticle(articleContent);

        if (t7_readability_js.curPageNum === 1) {
            var newNode = document.createElement('div');
            newNode.id = "readability-page-1";
            newNode.setAttribute("class", "page");
            t7_readability_js.moveNodeInnards(articleContent, newNode);
            articleContent.appendChild(newNode);
        }

        /**
         * Now that we've gone through the full algorithm, check to see if we got any meaningful content.
         * If we didn't, we may need to re-run grabArticle with different flags set. This gives us a higher
         * likelihood of finding the content, and the sieve approach gives us a higher likelihood of
         * finding the -right- content.
        **/
        if(t7_readability_js.getInnerText(articleContent, false).length < 250) {
            /*
            if (t7_readability_js.flagIsActive(t7_readability_js.FLAG_STRIP_UNLIKELYS)) {
                t7_readability_js.removeFlag(t7_readability_js.FLAG_STRIP_UNLIKELYS);
                t7_readability_js.clonedBody = document.body.cloneNode(true);
                return t7_readability_js.grabArticle();
            }
            else if (t7_readability_js.flagIsActive(t7_readability_js.FLAG_WEIGHT_CLASSES)) {
                t7_readability_js.removeFlag(t7_readability_js.FLAG_WEIGHT_CLASSES);
                t7_readability_js.clonedBody = document.body.cloneNode(true);
                return t7_readability_js.grabArticle();
            }
            else if (t7_readability_js.flagIsActive(t7_readability_js.FLAG_CLEAN_CONDITIONALLY)) {
                t7_readability_js.removeFlag(t7_readability_js.FLAG_CLEAN_CONDITIONALLY);
                t7_readability_js.clonedBody = document.body.cloneNode(true);
                return t7_readability_js.grabArticle();
            } else */{
                return null;
            }
        }
        return articleContent;
    },

    /**
     * Removes script tags from the document.
     *
     * @param Element
    **/
    removeScripts: function (doc) {
        /**
         * Don't remove scripts as it will break the functionality of the original page.
        var scripts = doc.getElementsByTagName('script');
        for(var i = scripts.length-1; i >= 0; i-=1)
        {
            if(typeof(scripts[i].src) === "undefined" || (scripts[i].src.indexOf('readability') === -1 && scripts[i].src.indexOf('typekit') === -1))
            {
                scripts[i].nodeValue="";
                scripts[i].removeAttribute('src');
                if (scripts[i].parentNode) {
                        scripts[i].parentNode.removeChild(scripts[i]);
                }
            }
        }
       **/
    },

    /**
     * Get the inner text of a node - cross browser compatibly.
     * This also strips out any excess whitespace to be found.
     *
     * @param Element
     * @return string
    **/
    getInnerText: function (e, normalizeSpaces) {
        var textContent    = "";

        if(typeof(e.textContent) === "undefined" && typeof(e.innerText) === "undefined") {
            return "";
        }

        normalizeSpaces = (typeof normalizeSpaces === 'undefined') ? true : normalizeSpaces;

        if (navigator.appName === "Microsoft Internet Explorer") {
            textContent = e.innerText.replace( t7_readability_js.regexps.trim, "" ); }
        else {
            textContent = e.textContent.replace( t7_readability_js.regexps.trim, "" ); }

        if(normalizeSpaces) {
            return textContent.replace( t7_readability_js.regexps.normalize, " "); }
        else {
            return textContent; }
    },

    /**
     * Get the number of times a string s appears in the node e.
     *
     * @param Element
     * @param string - what to split on. Default is ","
     * @return number (integer)
    **/
    getCharCount: function (e,s) {
        s = s || ",";
        return t7_readability_js.getInnerText(e).split(s).length-1;
    },

    /**
     * Remove the style attribute on every e and under.
     * TODO: Test if getElementsByTagName(*) is faster.
     *
     * @param Element
     * @return void
    **/
    cleanStyles: function (e) {
        e = e || document;
        var cur = e.firstChild;

        if(!e) {
            return; }

        // Remove any root styles, if we're able.
        if(typeof e.removeAttribute === 'function' && e.className !== 'readability-styled') {
            e.removeAttribute('style'); }

        // Go until there are no more child nodes
        while ( cur !== null ) {
            if ( cur.nodeType === 1 ) {
                // Remove style attribute(s) :
                if(cur.className !== "readability-styled") {
                    cur.removeAttribute("style");
                }
                t7_readability_js.cleanStyles( cur );
            }
            cur = cur.nextSibling;
        }
    },

    /**
     * Get the density of links as a percentage of the content
     * This is the amount of text that is inside a link divided by the total text in the node.
     *
     * @param Element
     * @return number (float)
    **/
    getLinkDensity: function (e) {
        var textLength = t7_readability_js.getInnerText(e).length;
        if (textLength === 0)
            return 0;
        var links      = e.getElementsByTagName("a");
        var linkLength = 0;
        for(var i=0, il=links.length; i<il;i+=1)
        {
            linkLength += t7_readability_js.getInnerText(links[i]).length;
        }

        return linkLength / textLength;
    },

    /**
     * Find a cleaned up version of the current URL, to use for comparing links for possible next-pageyness.
     *
     * @author Dan Lacy
     * @return string the base url
    **/
    findBaseUrl: function () {
        var noUrlParams     = window.location.pathname.split("?")[0],
            urlSlashes      = noUrlParams.split("/").reverse(),
            cleanedSegments = [],
            possibleType    = "";

        for (var i = 0, slashLen = urlSlashes.length; i < slashLen; i+=1) {
            var segment = urlSlashes[i];

            // Split off and save anything that looks like a file type.
            if (segment.indexOf(".") !== -1) {
                possibleType = segment.split(".")[1];

                /* If the type isn't alpha-only, it's probably not actually a file extension. */
                if(!possibleType.match(/[^a-zA-Z]/)) {
                    segment = segment.split(".")[0];
                }
            }

            /**
             * EW-CMS specific segment replacement. Ugly.
             * Example: http://www.ew.com/ew/article/0,,20313460_20369436,00.html
            **/
            if(segment.indexOf(',00') !== -1) {
                segment = segment.replace(',00', '');
            }

            // If our first or second segment has anything looking like a page number, remove it.
            if (segment.match(/((_|-)?p[a-z]*|(_|-))[0-9]{1,2}$/i) && ((i === 1) || (i === 0))) {
                segment = segment.replace(/((_|-)?p[a-z]*|(_|-))[0-9]{1,2}$/i, "");
            }


            var del = false;

            /* If this is purely a number, and it's the first or second segment, it's probably a page number. Remove it. */
            if (i < 2 && segment.match(/^\d{1,2}$/)) {
                del = true;
            }

            /* If this is the first segment and it's just "index", remove it. */
            if(i === 0 && segment.toLowerCase() === "index") {
                del = true;
            }


            /* If our first or second segment is smaller than 3 characters, and the first segment was purely alphas, remove it. */
            if(i < 2 && segment.length < 3 && !urlSlashes[0].match(/[a-z]/i)) {
                del = true;
            }

            /* If it's not marked for deletion, push it to cleanedSegments. */
            if (!del) {
                cleanedSegments.push(segment);
            }
        }

        // This is our final, cleaned, base article URL.
        return window.location.protocol + "//" + window.location.host + cleanedSegments.reverse().join("/");
    },

    /**
     * Look for any paging links that may occur within the document.
     *
     * @param body
     * @return object (array)
    **/
    findNextPageLink: function (elem) {
        var possiblePages = {},
            allLinks = elem.getElementsByTagName('a'),
            articleBaseUrl = t7_readability_js.findBaseUrl();

        /**
         * Loop through all links, looking for hints that they may be next-page links.
         * Things like having "page" in their textContent, className or id, or being a child
         * of a node with a page-y className or id.
         *
         * Also possible: levenshtein distance? longest common subsequence?
         *
         * After we do that, assign each page a score, and
        **/
        for(var i = 0, il = allLinks.length; i < il; i+=1) {
            var link     = allLinks[i],
                linkHref = allLinks[i].href.replace(/#.*$/, '').replace(/\/$/, '');

            /* If we've already seen this page, ignore it */
            if(linkHref === "" || linkHref === articleBaseUrl || linkHref === window.location.href || linkHref in t7_readability_js.parsedPages) {
                continue;
            }

            /* If it's on a different domain, skip it. */
            if(window.location.host !== linkHref.split(/\/+/g)[1]) {
                continue;
            }

            var linkText = t7_readability_js.getInnerText(link);

            /* If the linkText looks like it's not the next page, skip it. */
            if(linkText.match(t7_readability_js.regexps.extraneous) || linkText.length > 25) {
                continue;
            }

            /* If the leftovers of the URL after removing the base URL don't contain any digits, it's certainly not a next page link. */
            var linkHrefLeftover = linkHref.replace(articleBaseUrl, '');
            if(!linkHrefLeftover.match(/\d/)) {
                continue;
            }

            if(!(linkHref in possiblePages)) {
                possiblePages[linkHref] = {"score": 0, "linkText": linkText, "href": linkHref};
            } else {
                possiblePages[linkHref].linkText += ' | ' + linkText;
            }

            var linkObj = possiblePages[linkHref];

            /**
             * If the articleBaseUrl isn't part of this URL, penalize this link. It could still be the link, but the odds are lower.
             * Example: http://www.actionscript.org/resources/articles/745/1/JavaScript-and-VBScript-Injection-in-ActionScript-3/Page1.html
            **/
            if(linkHref.indexOf(articleBaseUrl) !== 0) {
                linkObj.score -= 25;
            }

            var linkData = linkText + ' ' + link.className + ' ' + link.id;
            if(linkData.match(t7_readability_js.regexps.nextLink)) {
                linkObj.score += 50;
            }
            if(linkData.match(/pag(e|ing|inat)/i)) {
                linkObj.score += 25;
            }
            if(linkData.match(/(first|last)/i)) { // -65 is enough to negate any bonuses gotten from a > or » in the text,
                /* If we already matched on "next", last is probably fine. If we didn't, then it's bad. Penalize. */
                if(!linkObj.linkText.match(t7_readability_js.regexps.nextLink)) {
                    linkObj.score -= 65;
                }
            }
            if(linkData.match(t7_readability_js.regexps.negative) || linkData.match(t7_readability_js.regexps.extraneous)) {
                linkObj.score -= 50;
            }
            if(linkData.match(t7_readability_js.regexps.prevLink)) {
                linkObj.score -= 200;
            }

            /* If a parentNode contains page or paging or paginat */
            var parentNode = link.parentNode,
                positiveNodeMatch = false,
                negativeNodeMatch = false;
            while(parentNode) {
                var parentNodeClassAndId = parentNode.className + ' ' + parentNode.id;
                if(!positiveNodeMatch && parentNodeClassAndId && parentNodeClassAndId.match(/pag(e|ing|inat)/i)) {
                    positiveNodeMatch = true;
                    linkObj.score += 25;
                }
                if(!negativeNodeMatch && parentNodeClassAndId && parentNodeClassAndId.match(t7_readability_js.regexps.negative)) {
                    /* If this is just something like "footer", give it a negative. If it's something like "body-and-footer", leave it be. */
                    if(!parentNodeClassAndId.match(t7_readability_js.regexps.positive)) {
                        linkObj.score -= 25;
                        negativeNodeMatch = true;
                    }
                }

                parentNode = parentNode.parentNode;
            }

            /**
             * If the URL looks like it has paging in it, add to the score.
             * Things like /page/2/, /pagenum/2, ?p=3, ?page=11, ?pagination=34
            **/
            if (linkHref.match(/p(a|g|ag)?(e|ing|ination)?(=|\/)[0-9]{1,2}/i) || linkHref.match(/(page|paging)/i)) {
                linkObj.score += 25;
            }

            /* If the URL contains negative values, give a slight decrease. */
            if (linkHref.match(t7_readability_js.regexps.extraneous)) {
                linkObj.score -= 15;
            }

            /**
             * Minor punishment to anything that doesn't match our current URL.
             * NOTE: I'm finding this to cause more harm than good where something is exactly 50 points.
             *       Dan, can you show me a counterexample where this is necessary?
             * if (linkHref.indexOf(window.location.href) !== 0) {
             *    linkObj.score -= 1;
             * }
            **/

            /**
             * If the link text can be parsed as a number, give it a minor bonus, with a slight
             * bias towards lower numbered pages. This is so that pages that might not have 'next'
             * in their text can still get scored, and sorted properly by score.
            **/
            var linkTextAsNumber = parseInt(linkText, 10);
            if(linkTextAsNumber) {
                // Punish 1 since we're either already there, or it's probably before what we want anyways.
                if (linkTextAsNumber === 1) {
                    linkObj.score -= 10;
                }
                else {
                    // Todo: Describe this better
                    linkObj.score += Math.max(0, 10 - linkTextAsNumber);
                }
            }
        }

        /**
         * Loop thrugh all of our possible pages from above and find our top candidate for the next page URL.
         * Require at least a score of 50, which is a relatively high confidence that this page is the next link.
        **/
        var topPage = null;
        for(var page in possiblePages) {
            if(possiblePages.hasOwnProperty(page)) {
                if(possiblePages[page].score >= 50 && (!topPage || topPage.score < possiblePages[page].score)) {
                    topPage = possiblePages[page];
                }
            }
        }

        if(topPage) {
            var nextHref = topPage.href.replace(/\/$/,'');

            t7_readability_js.dbg('NEXT PAGE IS ' + nextHref);
            t7_readability_js.parsedPages[nextHref] = true;
            return nextHref;
        }
        else {
            return null;
        }
    },

    createLinkDiv: function(link) {
        var divNode = document.createElement('div');
        var aNode = document.createElement('a');
        var tNode = document.createTextNode('View Next Page');
        divNode.setAttribute('style', 'text-align: center');
        aNode.setAttribute('href', link);
        aNode.appendChild(tNode);
        divNode.appendChild(aNode);
        return divNode;
    },

    xhr: function () {
        if (typeof XMLHttpRequest !== 'undefined' && (window.location.protocol !== 'file:' || !window.ActiveXObject)) {
            return new XMLHttpRequest();
        }
        else {
            try { return new ActiveXObject('Msxml2.XMLHTTP.6.0'); } catch(sixerr) { }
            try { return new ActiveXObject('Msxml2.XMLHTTP.3.0'); } catch(threrr) { }
            try { return new ActiveXObject('Msxml2.XMLHTTP'); } catch(err) { }
        }

        return false;
    },

    successfulRequest: function (request) {
        return (request.status >= 200 && request.status < 300) || request.status === 304 || (request.status === 0 && request.responseText);
    },

    ajax: function (url, options) {
        var request = t7_readability_js.xhr();

        function respondToReadyState(readyState) {
            if (request.readyState === 4) {
                if (t7_readability_js.successfulRequest(request)) {
                    if (options.success) { options.success(request); }
                }
                else {
                    if (options.error) { options.error(request); }
                }
            }
        }

        if (typeof options === 'undefined') { options = {}; }

        request.onreadystatechange = respondToReadyState;

        request.open('get', url, true);
        request.setRequestHeader('Accept', 'text/html');

        try {
            request.send(options.postBody);
        }
        catch (e) {
            if (options.error) { options.error(); }
        }

        return request;
    },

    /**
     * Make an AJAX request for each page and append it to the document.
    **/
    curPageNum: 1,

    appendNextPage: function (nextPageLink) {
        t7_readability_js.curPageNum+=1;

        var articlePage       = document.createElement("DIV");
        articlePage.id        = 'readability-page-' + t7_readability_js.curPageNum;
        articlePage.className = 'page';
        articlePage.innerHTML = '<p class="page-separator" title="Page ' + t7_readability_js.curPageNum + '">&sect;</p>';

        document.getElementById("readability-content").appendChild(articlePage);

        if(t7_readability_js.curPageNum > t7_readability_js.maxPages) {
            var linkDiv = t7_readability_js.createLinkDiv(nextPageLink);

            articlePage.appendChild(linkDiv);
            return;
        }

        /**
         * Now that we've built the article page DOM element, get the page content
         * asynchronously and load the cleaned content into the div we created for it.
        **/
        (function(pageUrl, thisPage) {
            t7_readability_js.ajax(pageUrl, {
                success: function(r) {

                    /* First, check to see if we have a matching ETag in headers - if we do, this is a duplicate page. */
                    var eTag = r.getResponseHeader('ETag');
                    if(eTag) {
                        if(eTag in t7_readability_js.pageETags) {
                            t7_readability_js.dbg("Exact duplicate page found via ETag. Aborting.");
                            articlePage.style.display = 'none';
                            return;
                        } else {
                            t7_readability_js.pageETags[eTag] = 1;
                        }
                    }

                    // TODO: this ends up doubling up page numbers on NYTimes articles. Need to generically parse those away.
                    var page = document.createElement("DIV");

                    /**
                     * Do some preprocessing to our HTML to make it ready for appending.
                     * • Remove any script tags. Swap and reswap newlines with a unicode character because multiline regex doesn't work in javascript.
                     * • Turn any noscript tags into divs so that we can parse them. This allows us to find any next page links hidden via javascript.
                     * • Turn all double br's into p's - was handled by prepDocument in the original view.
                     *   Maybe in the future abstract out prepDocument to work for both the original document and AJAX-added pages.
                    **/
                    var pageInnards = r.responseXML;
                    t7_readability_js.removeScripts(pageInnards);
                    t7_readability_js.replaceNoscriptsWithPs(pageInnards);
                    t7_readability_js.replaceDoubleBrsWithPs(pageInnards);
                    t7_readability_js.replaceFontsWithSpans(pageInnards);
                    page.appendChild(pageInnards);


                    /**
                     * Reset all flags for the next page, as they will search through it and disable as necessary at the end of grabArticle.
                    **/
                    t7_readability_js.flags = 0x1 | 0x2 | 0x4;

                    var nextPageLink = t7_readability_js.findNextPageLink(page),
                        content      =  t7_readability_js.grabArticle(page, true);

                    if(!content) {
                        t7_readability_js.dbg("No content found in page to append. Aborting.");
                        return;
                    }

                    /**
                     * Anti-duplicate mechanism. Essentially, get the first paragraph of our new page.
                     * Compare it against all of the the previous document's we've gotten. If the previous
                     * document contains exactly the innerHTML of this first paragraph, it's probably a duplicate.
                    **/
                    var firstP = content.getElementsByTagName("P").length ? content.getElementsByTagName("P")[0] : null;
                    if(firstP && firstP.innerHTML.length > 100) {
                        for(var i=1; i <= t7_readability_js.curPageNum; i+=1) {
                            var rPage = document.getElementById('readability-page-' + i);
                            if(rPage && rPage.innerHTML.indexOf(firstP.innerHTML) !== -1) {
                                t7_readability_js.dbg('Duplicate of page ' + i + ' - skipping.');
                                articlePage.style.display = 'none';
                                t7_readability_js.parsedPages[pageUrl] = true;
                                return;
                            }
                        }
                    }

                    t7_readability_js.removeScripts(content);

                    t7_readability_js.moveNodeInnards(content, thisPage);

                    /**
                     * After the page has rendered, post process the content. This delay is necessary because,
                     * in webkit at least, offsetWidth is not set in time to determine image width. We have to
                     * wait a little bit for reflow to finish before we can fix floating images.
                    **/
                    window.setTimeout(
                        function() { t7_readability_js.postProcessContent(thisPage); },
                        500
                    );

                    if(nextPageLink) {
                        t7_readability_js.appendNextPage(nextPageLink);
                    }
                }
            });
        }(nextPageLink, articlePage));
    },

    /**
     * Get an elements class/id weight. Uses regular expressions to tell if this
     * element looks good or bad.
     *
     * @param Element
     * @return number (Integer)
    **/
    getClassWeight: function (e) {
        if(!t7_readability_js.flagIsActive(t7_readability_js.FLAG_WEIGHT_CLASSES)) {
            return 0;
        }
        var weight = 0;

        /* Look for a special classname */
        if (typeof(e.className) === 'string' && e.className !== '')
        {
            if(e.className.search(t7_readability_js.regexps.ads) !== -1) {
                t7_readability_js.dbg("Found ads in className");
                weight -= 10000; }

            if(e.className.search(t7_readability_js.regexps.negative) !== -1) {
                weight -= 25; }

            if(e.className.search(t7_readability_js.regexps.positive) !== -1) {
                weight += 25; }
        }

        /* Look for a special ID */
        if (typeof(e.id) === 'string' && e.id !== '')
        {
            if(e.id.search(t7_readability_js.regexps.ads) !== -1) {
                t7_readability_js.dbg("Found ads in id");
                weight -= 10000; }

            if(e.id.search(t7_readability_js.regexps.negative) !== -1) {
                weight -= 25; }

            if(e.id.search(t7_readability_js.regexps.positive) !== -1) {
                weight += 25; }
        }

        return weight;
    },

    getOriginalNode: function (node) {
        if (node.id)
            return t7_readability_js.document.getElementById(node.id);
        return null;
    },

    nodeIsVisible: function (node, isCloned) {
        if (!node || node.tagName === "SCRIPT" || node.tagName === 'NOSCRIPT' || node.tagName === "STYLE")
            return false;

        if (node.nodeType === Node.TEXT_NODE) {
            node = node.parentNode;
        }

        if (typeof node.visible !== 'undefined')
            return node.visible;

        if (typeof isCloned === 'undefined')
            isCloned = true;

        var originalNode = isCloned ? t7_readability_js.document.getElementById(node.id) : node;
        if (!originalNode) {
            if (node.tagName === 'BODY') {
                originalNode = document.body;
            } else {
                t7_readability_js.dbg('node=' + t7_readability_js.nodeInfo(node, false)
                    + ', id=' + node.id);
                throw 'originalNode is null';
            }
        }
        //t7_readability_js.dbg("  originalNode=" + t7_readability_js.nodeInfo(originalNode));

        var ret = false;

        try {
            var left = originalNode.offsetLeft, right = originalNode.offsetLeft + originalNode.offsetWidth;
            ret = (originalNode.offsetWidth !== 0 || originalNode.offsetHeight !== 0 || originalNode.tagName === 'BR')
                && (left >= 0 && left <= document.body.offsetWidth || right >= 0 && right <= document.body.offsetWidth)
                && getComputedStyle(originalNode, '').display !== 'none';
        } catch (e) {
            ret = isCloned && !originalNode;
        }
        if (!ret) {
            //t7_readability_js.dbg("  invisible node: " + t7_readability_js.nodeInfo(node) + ", isCloned: " + isCloned);
        }
        if (isCloned)
            node.visible = ret;
        return ret;
    },

    /**
     * Remove extraneous break tags from a node.
     *
     * @param Element
     * @return void
     **/
    killBreaks: function (e) {
        var allElements = e.getElementsByTagName('*');
        var i = 0;
        while (i < allElements.length) {
            t7_readability_js.deleteExtraBreaks(allElements[i]);
            i++;
        }
    },

    /**
     * Clean a node of all elements of type "tag".
     * (Unless it's a youtube/vimeo video. People love movies.)
     *
     * @param Element
     * @param string tag to clean
     * @return void
     **/
    clean: function (e, tag) {
        var targetList = e.getElementsByTagName( tag );
        var isEmbed    = (tag === 'object' || tag === 'embed');

        for (var y=targetList.length-1; y >= 0; y-=1) {
            /* Allow youtube and vimeo videos through as people usually want to see those. */
            if(isEmbed) {
                var attributeValues = "";
                for (var i=0, il=targetList[y].attributes.length; i < il; i+=1) {
                    attributeValues += targetList[y].attributes[i].value + '|';
                }

                /* First, check the elements attributes to see if any of them contain youtube or vimeo */
                if (attributeValues.search(t7_readability_js.regexps.videos) !== -1) {
                    continue;
                }

                /* Then check the elements inside this element for the same. */
                if (targetList[y].innerHTML.search(t7_readability_js.regexps.videos) !== -1) {
                    continue;
                }

            }

            targetList[y].parentNode.removeChild(targetList[y]);
        }
    },

    /**
     * Clean an element of all tags of type "tag" if they look fishy.
     * "Fishy" is an algorithm based on content length, classnames, link density, number of images & embeds, etc.
     *
     * @return void
     **/
    cleanConditionally: function (e, tag) {

        if(!t7_readability_js.flagIsActive(t7_readability_js.FLAG_CLEAN_CONDITIONALLY)) {
            return;
        }

        var tagsList      = e.getElementsByTagName(tag);
        var curTagsLength = tagsList.length;
        /**
         * Gather counts for other typical elements embedded within.
         * Traverse backwards so we can remove nodes at the same time without effecting the traversal.
         *
         * TODO: Consider taking into account original contentScore here.
        **/
        for (var i=curTagsLength-1; i >= 0; i-=1) {
            var weight = t7_readability_js.getClassWeight(tagsList[i]);
            var contentScore = (typeof tagsList[i].readability !== 'undefined') ? tagsList[i].readability.contentScore : 0;

            t7_readability_js.dbg("Cleaning Conditionally " + tagsList[i] + " (" + tagsList[i].className + ":" + tagsList[i].id + ")" + ((typeof tagsList[i].readability !== 'undefined') ? (" with score " + tagsList[i].readability.contentScore) : ''));

            if(weight+contentScore < 0)
            {
                tagsList[i].parentNode.removeChild(tagsList[i]);
            }
            else if ( t7_readability_js.getCharCount(tagsList[i],',') < 10) {
                /**
                 * If there are not very many commas, and the number of
                 * non-paragraph elements is more than paragraphs or other ominous signs, remove the element.
                **/
                var p      = tagsList[i].getElementsByTagName("p").length;
                var img    = tagsList[i].getElementsByTagName("img").length;
                var li     = tagsList[i].getElementsByTagName("li").length-100;
                var input  = tagsList[i].getElementsByTagName("input").length;
                var td     = tagsList[i].getElementsByTagName("td").length;

                var embedCount = 0;
                var embeds     = tagsList[i].getElementsByTagName("embed");
                for(var ei=0,il=embeds.length; ei < il; ei+=1) {
                    if (embeds[ei].src.search(t7_readability_js.regexps.videos) === -1) {
                      embedCount+=1;
                    }
                }

                var linkDensity   = t7_readability_js.getLinkDensity(tagsList[i]);
                var contentLength = t7_readability_js.getInnerText(tagsList[i]).length;
                var toRemove      = false;

                if ( img > p && img > 5) {
                    toRemove = true;
                } else if(li > p && tag !== "ul" && tag !== "ol") {
                    toRemove = true;
                } else if( input > Math.floor(p/3) ) {
                    toRemove = true;
                } else if(contentLength < 25 && (img === 0 || img > 2) ) {
                    toRemove = true;
                } else if(weight < 25 && linkDensity > 0.2) {
                    toRemove = true;
                } else if(weight >= 25 && linkDensity > 0.5) {
                    toRemove = true;
                } else if((embedCount === 1 && contentLength < 75) || embedCount > 1) {
                    toRemove = true;
                } else if(contentLength < td * 10) {
                    toRemove = true;
                }

                if(toRemove) {
                    tagsList[i].parentNode.removeChild(tagsList[i]);
                }
            }
        }
    },

    /**
     * Clean out spurious headers from an Element. Checks things like classnames and link density.
     *
     * @param Element
     * @return void
    **/
    cleanHeaders: function (e) {
        for (var headerIndex = 1; headerIndex < 3; headerIndex+=1) {
            var headers = e.getElementsByTagName('h' + headerIndex);
            for (var i=headers.length-1; i >=0; i-=1) {
                if (t7_readability_js.getClassWeight(headers[i]) < 0 || t7_readability_js.getLinkDensity(headers[i]) > 0.33) {
                    headers[i].parentNode.removeChild(headers[i]);
                }
            }
        }
    },

    flagIsActive: function(flag) {
        return (t7_readability_js.flags & flag) > 0;
    },

    addFlag: function(flag) {
        t7_readability_js.flags = t7_readability_js.flags | flag;
    },

    removeFlag: function(flag) {
        t7_readability_js.flags = t7_readability_js.flags & ~flag;
    },

    // Removes the children of |src| and appends them to |dest|.
    moveNodeInnards: function(src, dest) {
        try {
            while (src.firstChild) {
                dest.appendChild(src.removeChild(src.firstChild));
            }
        } catch (e) {}
    },

    // Returns true if the node is a whitespace text node.
    isWhitespaceNode: function(node) {
        if (node.nodeType == Node.TEXT_NODE) {
            if (node.data.trim().length == 0) {
               return true;
            }
        }
        return false;
    },

    // Returns true if the node is a <BR>.
    isBrNode: function(node) {
        return (node.tagName === 'BR');
    },


    // Returns the last <BR> node in a sequence of <BR> nodes that are only
    // separated by whitespace, or null if there are not at least two <BR> tags
    // in the sibling chain starting with |node|. Returns the second such <BR>
    // node if |restrictToTwo| is true.
    isMultipleBr: function(node, restrictToTwo) {
        var lastBr = null;
        if (!t7_readability_js.isBrNode(node)) {
            return lastBr;
        }
        var curr = node.nextSibling;
        while (curr && (t7_readability_js.isWhitespaceNode(curr) || t7_readability_js.isBrNode(curr))) {
            if (t7_readability_js.isBrNode(curr)) {
                if (restrictToTwo) {
                    return curr;
                } else {
                    lastBr = curr;
                }
            }
            curr = curr.nextSibling;
        }
        return lastBr;
    },

    // Removes all <BR> nodes except one and whitespace in between in a series
    // of <BR> nodes.
    deleteExtraBreaks: function(node) {
        var lastBr = t7_readability_js.isMultipleBr(node, false);
        var ret = false;
        while (lastBr && lastBr != node) {
            var toRemove = lastBr;
            lastBr = lastBr.previousSibling;
            toRemove.parentNode.removeChild(toRemove);
            ret = true;
        }
        return ret;
    },

    // Replaces a pair of <BR> nodes (possibly separated by whitespace), with a
    // <P> node, and makes all next siblings of that pair children of <P>, up
    // until the next pair of <BR> nodes is reached.
    replaceDoubleBrWithP: function(node) {
        // Check that we are starting with a BR.
        var second = t7_readability_js.isMultipleBr(node, true);
        if (!second) {
            return;
        }
        // Make all next siblings of the second BR into children of a P.
        var p = document.createElement('p');
        var curr = second.nextSibling;
        while (curr) {
            if (t7_readability_js.isMultipleBr(curr, true)) {
                break;
            }
            var next = curr.nextSibling;
            p.appendChild(curr.parentNode.removeChild(curr));
            curr = next;
        }
        var ret = curr;

        // Remove all nodes between the first and second BR.
        curr = node.nextSibling;
        while (curr && curr != second) {
            var next = curr.nextSibling;
            curr.parentNode.removeChild(curr);
            curr = next;
        }
        // Remove the second BR.
        second.parentNode.removeChild(second);
        // Replace the first BR with the P.
        // Avoid empty <p>, it will cause incorrect candidates in grabArticle().
        if (p.textContent.trim().length > 0) {
            p.visible = true;
            node.parentNode.replaceChild(p, node);
        }
        return ret;
    },

    // Returns true if the NodeList contains a double <BR>.
    hasDoubleBr: function(nodeList) {
        for (var i = 0; i < nodeList.length; i++) {
            if (t7_readability_js.isMultipleBr(nodeList[i], true)) {
                return true;
            }
        }
        return false;
    },

    // Replaces double <BR> tags with <P> tags.
    replaceDoubleBrsWithPs: function(node) {
        var allElements = node.getElementsByTagName('BR');
        var curr = null;
        while (allElements && allElements.length > 0 &&
               t7_readability_js.hasDoubleBr(allElements)) {
            for (var nodeIndex = 0; (curr = allElements[nodeIndex]); nodeIndex += 1) {
                var next = curr;
                while (next = t7_readability_js.replaceDoubleBrWithP(next));
            }
            allElements = node.getElementsByTagName('BR');
        }
    },


    // Replaces a BR and the whitespace that follows it with a P.
    replaceBrWithP: function(node) {
        if (!t7_readability_js.isBrNode(node)) {
            return;
        }
        var p = document.createElement('p');
        var curr = node.nextSibling;
        while (curr && !t7_readability_js.isBrNode(curr)) {
            var next = curr.nextSibling;
            if (t7_readability_js.isWhitespaceNode(curr)) {
                curr.parentNode.removeChild(curr);
            } else {
                p.appendChild(curr.parentNode.removeChild(curr));
            }
            curr = next;
        }
        p.visible = true;
        node.parentNode.replaceChild(p, node);
        return curr;
    },

    // Replaces all <BR> tags with <P> tags. Makes all next siblings of a <BR> tag
    // children of the <P>.
    replaceBrsWithPs: function(node) {
        var allElements = node.getElementsByTagName('BR');
        var curr = null;
        while (allElements && allElements.length > 0) {
            for (var nodeIndex = 0; (curr = allElements[nodeIndex]); nodeIndex += 1) {
                var parent = curr.parentNode;
                var lastParent = parent;
                if (curr === parent.firstChild && parent.tagName === "SPAN") {
                    // Move <BR> to the top-most none <SPAN> ancestral node.
                    while (parent.tagName === "SPAN") {
                        lastParent = parent;
                        parent = parent.parentNode;
                    }
                    curr = parent.insertBefore(curr.parentNode.removeChild(curr), lastParent);
                }
                var next = curr;
                while (next = t7_readability_js.replaceBrWithP(next));
            }
            allElements = node.getElementsByTagName('BR');
        }
    },

    // Replaces any tag with any other tag.
    replaceTagsWithTags: function(node, srcTag, destTag) {
        var allElements = node.getElementsByTagName(srcTag);
        for (var i = 0; i < allElements.length; i++) {
            var dest = document.createElement(destTag);
            t7_readability_js.moveNodeInnards(allElements[i], dest);
            dest.id = allElements[i].id;
            allElements[i].parentNode.replaceChild(dest, allElements[i]);
        }
    },

    // Replaces all <noscript> tags with <p> tags.
    replaceNoscriptsWithPs: function(node) {
        t7_readability_js.replaceTagsWithTags(node, 'noscript', 'p');
    },

    // Replaces all <font> tags with <span> tags.
    replaceFontsWithSpans: function(node) {
        t7_readability_js.replaceTagsWithTags(node, 'font', 'span');
    },

    // Returns a list of image URLs in the distilled article.
    getImages : function() {
        var images = document.getElementsByTagName('img');
        var result = new Array(images.length);
        t7_readability_js.dbg("Number of images: " + images.length);
        for(i = 0; i < images.length; i++) {
            result[i] = images[i].src;
            t7_readability_js.dbg("Image: " + result[i]);
        }
        return result;
    },

    // Returns the distilled article HTML from the page(s).
    getDistilledArticleHTML : function() {
        return t7_readability_js.distilledHTML;
    },

    // Returns the next page of this article.
    getNextPageLink : function() {
        return t7_readability_js.nextPageLink;
    },

    idPrefix: 'bdb7rdblt',
    assignIdForElements: function() {
        if(!t7_readability_js.document || !t7_readability_js.document.body) return;

        var index = 0;
        var elements = t7_readability_js.document.body.getElementsByTagName('*');
        var ids = [];
        for(var i = 0, element = elements[i]; i < elements.length; element = elements[++i]) {
            if (element.id) {
                element.origId = element.id;
                if (ids.indexOf(element.id) < 0) {
                    ids.push(element.id);
                    continue;
                }
            }
            element.id = (t7_readability_js.idPrefix + ++index);
        }
    },

    removeIdForElements: function(articleContent) {
        if(!t7_readability_js.document || !t7_readability_js.document.body) return;

        var elements = t7_readability_js.document.body.getElementsByTagName('*');
        for(var i = 0, element = elements[i]; i < elements.length; element = elements[++i]) {
            //t7_readability_js.dbg('Removing id for:' + t7_readability_js.nodeInfo(element));
            if (element.id && (typeof element.id === 'string')) {
                if (element.origId)
                    element.id = element.origId;
                else {
                    element.id = undefined;
                    element.attributes.removeNamedItem('id');
                }
            }
        }

        if (!articleContent) return;

        elements = articleContent.getElementsByTagName('*');
        for(var i = 0, element = elements[i]; i < elements.length; element = elements[++i]) {
            if (element.id && (typeof element.id === 'string')) {
                if (element.origId)
                    element.id = element.origId;
                else {
                    element.id = undefined;
                    element.attributes.removeNamedItem('id');
                }
            }
        }
    },

    // Get first block of readable text from the distilled content.
    getFirstText: function (e, threshold) {
        if (e.nodeType === Node.TEXT_NODE
            && e.data.replace(/\s/g, '').length > threshold
            && !t7_readability_js.isOverflowHidden(
                document.getElementById(e.parentNode.id || e.parentNode.parentNode.id))) {
            return e.data;
        }
        for (var i = 0; i < e.childNodes.length; i += 1) {
            var resultChild = t7_readability_js.getFirstText(e.childNodes[i], threshold);
            if (null != resultChild)
                return resultChild;
        }
        return null;
    },

    getParentNodeWithMoreThanOneChild: function (e) {
        var current = e;
        var contentLength = t7_readability_js.distilledArticleContent.textContent.length;
        while (current && current.tagName !== 'P' && current.parentElement.childElementCount == 1)
            current = current.parentElement;
        return current.tagName === 'P' ? current
            : (current.textContent.length > contentLength * 0.8 ? current : current.parentElement);
    },

    // Search the current(un-distlled) document for the parent element of
    // the element contains the given text.
    getParentElementOfText: function (text) {
        var allElements = document.body.getElementsByTagName("*");
        for (var i = 0; i < allElements.length; i += 1) {
            if (!t7_readability_js.nodeIsVisible(allElements[i], false))
                continue;
            for (var j = 0; j < allElements[i].childNodes.length; j+=1) {
                var node = allElements[i].childNodes[j];
                if (node.nodeType == Node.TEXT_NODE
                    && node.data.indexOf(text) >=0) {
                    return t7_readability_js.getParentNodeWithMoreThanOneChild(allElements[i]);
                }
            }
        }
        return null;
    },

    // Get the client rect of the readable content in the current document.
    getButtonRect: function () {
        if (null == t7_readability_js.containerElement) {
            if (t7_readability_js.distilledArticleContent) {
                // Get the parent element of the first text node in distilled content in the original document.
                var threshold = t7_readability_js.THRESHOLD_SUBJECT_LENGTH;
                var firstText = t7_readability_js.getFirstText(t7_readability_js.distilledArticleContent, threshold);
                while (!firstText && threshold > 10) {
                    threshold /= 2;
                    firstText = t7_readability_js.getFirstText(t7_readability_js.distilledArticleContent, threshold);
                }
                if (firstText) {
                    t7_readability_js.containerElement = t7_readability_js.getParentElementOfText(firstText);
                }
            }
        } else if (t7_readability_js.containerElement.getClientRects().length == 0) {
            t7_readability_js.containerElement.visible = false;
            return null;
        } else if (t7_readability_js.isOverflowHidden(t7_readability_js.containerElement)) {
            t7_readability_js.hasOverflowHidden = false;
            t7_readability_js.containerElement.visible = false;
            return null;
        }

        if (null == t7_readability_js.containerElement) {
            return null;
        }

        // t7_readability_js.containerElement.style.boxShadow = '0 0 12px red';
        var cr = t7_readability_js.containerElement.getBoundingClientRect();

        // Calculate actual zoom factor of containerElement.
        var zoom = 1;
        var c = t7_readability_js.containerElement;
        while (c) {
            zoom *= getComputedStyle(c, '').zoom;
            // t7_readability_js.dbg('  [' + t7_readability_js.nodeInfo(c) + '] zoom=' + zoom);
            c = c.parentElement;
        }
        t7_readability_js.dbg('  zoom=' + zoom);

        // Ammend the scoll offset and zoom factor.
        t7_readability_js.buttonRect = {
            left: cr.right  * zoom + window.scrollX - t7_readability_js.getScaledButtonWidth(),
            top: cr.top     * zoom + window.scrollY,
            right: cr.right * zoom + window.scrollX,
            bottom: cr.top  * zoom + window.scrollY + t7_readability_js.getScaledButtonWidth()};
        return t7_readability_js.buttonRect;
    },

    hasOverflowHidden: false,
    isOverflowHidden: function (node) {
        // t7_readability_js.dbg('isOverflowHidden: ' + node.outerHTML);

        // if (typeof node.overflowHidden !== 'undefined')
        //     return node.overflowHidden;
        function isIntersecting(a, b) {
            if (a.left >= b.right
                || a.right <= b.left) {
                return false;
            }
            if (a.bottom <= b.top
                || a.top >= b.bottom) {
                return false;
            }
            return true;
        }
        var p = node.parentElement;
        while (p) {
            if (getComputedStyle(p, '').overflow === 'hidden') {
                if (!isIntersecting(node.getBoundingClientRect(), p.getBoundingClientRect())) {
                    node.overflowHidden = true;
                    t7_readability_js.hasOverflowHidden = true;
                    // t7_readability_js.dbg('node is overflow hidden by ' + p.outerHTML);
                    return true;
                }
            }
            p = p.parentElement;
        }
        node.overflowHidden = false;
        return false;
    },

    rescan: function() {
        t7_readability_js.distilledHTML = null;
        t7_readability_js.distilledArticleContent = null;
        t7_readability_js.containerElement = null;
        t7_readability_js.init();
        if (typeof(window.t7_subjectView_obj) != "undefined") {
            window.t7_subjectView_obj.updateSubject(t7_readability_js.getArticleTitle(),
            t7_readability_js.distilledHTML);
        }
    },

    subjectElement: null,
    subjectButtonVisible: false,

    // Draw the "reader" button at the left top corner of the readable content
    // in current page.
    drawSubjectButton: function() {
        var img = document.createElement("img");
        img.src = t7_readability_js.IMG_BASE64;
        img.style.width = "100%";
        img.style.height = "100%";
        var span = document.createElement("span");
        //span.style.background = "url(" + t7_readability_js.IMG_BASE64 + ") no-repeat";
        span.style.width = "100%";
        span.style.height = "100%";
        span.style.display = "inline-block";
        span.appendChild(img);

        var div = document.createElement("div");
        div.id = t7_readability_js.SUBJECT_ID;
        div.style.position = "absolute";
        div.onclick = t7_readability_js.showDistilledContent;
        div.onmouseenter = function() {this.style.opacity = "0.8";};
        div.onmouseleave = function() {this.style.opacity = "0.5";};
        div.style.opacity = "0.5";
        div.style.zIndex = 10000;

        t7_readability_js.subjectElement = div;
        t7_readability_js.subjectButtonVisible = true;
        if (t7_readability_js.adjustButtonSizeAndPosition()) {
            div.appendChild(span);
            document.documentElement.appendChild(div);
        } else if (!t7_readability_js.subjectElement.parentNode) {
            t7_readability_js.subjectElement = null;
        }
    },

    retried: false,

    adjustButtonSizeAndPosition: function() {
        // Adjust button position and size on zoom/rotate.
        var div = t7_readability_js.subjectElement;
        var rect = t7_readability_js.getButtonRect();
        if (!rect) {
            if (t7_readability_js.hasOverflowHidden) {
                t7_readability_js.dbg('Distilled contents are all overflow hidden, dropping');
                // Distilled text are all in overflown hidden elements, drop them all.
                t7_readability_js.distilledArticleContent = null;
                t7_readability_js.distilledHTML = '';
                return false;
            }
            div.style.display = 'none';
            t7_readability_js.subjectButtonVisible = false;
            if (t7_readability_js.retried) {
                t7_readability_js.dbg("No valid containerElement in second try, giving up");
                return false;
            }
            t7_readability_js.dbg("No valid containerElement, retrying...");
            t7_readability_js.retried = true;
            t7_readability_js.rescan();
            t7_readability_js.retried = false;
            return false;
        }
        div.style.width = t7_readability_js.getScaledButtonWidth() + "px";
        div.style.height = div.style.width;
        div.style.left = rect.left + "px";
        div.style.top = rect.top + "px";
        return true;
    },

    /**
     * Show distilled page content in current window/tab.
    **/
    showDistilledContent: function(type) {
        type = type || 2;
        if (t7_readability_js.distilledHTML) {
            if (type == 1) {
                document.documentElement.innerHTML = "<head><meta charset=\"utf-8\" ><meta name=\"viewport\" content=\"width=device-width, minimum-scale=1, user-scalable=no\"/>"
                        + "<title>" + t7_readability_js.getArticleTitle() + "</title><link rel=\"stylesheet\" type=\"text/css\" href=\"http://172.22.133.135/home/readability/css/t7_readability_js.css\"></head>"
                        + "<body><div id=\"mainContent\"><div id=\"article\">"
                        + "<article><header><h1>" + t7_readability_js.getArticleTitle() + "</h1></header><div id=\"content\">" + t7_readability_js.distilledHTML + "</div></article>"
                        + "</div></div></body>";
                return true;
            } else {
                if (typeof(window.t7_subjectView_obj) != "undefined") {
                    window.t7_subjectView_obj.show();
                    t7_readability_js.dbg("OK: window.t7_subjectView_obj.show() called!");
                } else {
                    t7_readability_js.dbg("ERROR: window.t7_subjectView_obj is undefined!");
                    //return t7_readability_js.showDistilledContent(1);
                }
            }
        }
        return false;
    },

    /**
     * Restore original page content in current window/tab.
    **/
    restoreContent: function() {
        if (t7_readability_js.originalHTML) {
            document.documentElement.innerHTML = t7_readability_js.originalHTML;
            return true;
        }
        return false;
    },

    scaleFactor: 1,
    getScaledButtonWidth: function() {
        var width = Math.round(t7_readability_js.MAGNIFIER_BUTTON_WIDTH / t7_readability_js.scaleFactor * 0.7);
        t7_readability_js.dbg('scaled button width: ' + width);
        return width;
    },

    getZoomLevel: function (newScale) {
        var screenCssPixelRatio = newScale || (window.outerWidth / window.innerWidth);
        //t7_readability_js.dbg("css pixel ratio=" + screenCssPixelRatio);
        var zoomLevel = "-6";
        if (screenCssPixelRatio < .36) {
        } else if (screenCssPixelRatio <= .41) {
          zoomLevel = "-5";
        } else if (screenCssPixelRatio <= .54) {
          zoomLevel = "-4";
        } else if(screenCssPixelRatio <= .64) {
          zoomLevel = "-3";
        } else if (screenCssPixelRatio <= .76) {
          zoomLevel = "-2";
        } else if (screenCssPixelRatio <= .92) {
          zoomLevel = "-1";
        } else if (screenCssPixelRatio <= 1.10) {
          zoomLevel = "0";
        } else if (screenCssPixelRatio <= 1.32) {
          zoomLevel = "1";
        } else if (screenCssPixelRatio <= 1.58) {
          zoomLevel = "2";
        } else if (screenCssPixelRatio <= 1.90) {
          zoomLevel = "3";
        } else if (screenCssPixelRatio <= 2.28) {
          zoomLevel = "4";
        } else if (screenCssPixelRatio <= 2.70) {
          zoomLevel = "5";
        } else {
          zoomLevel = "6";
        }
        t7_readability_js.dbg("zoom level=" + zoomLevel);
        t7_readability_js.scaleFactor = screenCssPixelRatio;
        return zoomLevel;
    },

    checkZoomLevel: function(newScale) {
        // valve value: 0 if auto-sizing if disabled, -3 elsewise.
        var shouldShowButton = (t7_readability_js.getZoomLevel(newScale) < 0);
        if (shouldShowButton && !t7_readability_js.subjectElement) {
            t7_readability_js.drawSubjectButton();
            return;
        }

        if (shouldShowButton != t7_readability_js.subjectButtonVisible) {
            t7_readability_js.subjectElement.style.display =
                shouldShowButton ? "block" : "none";
            t7_readability_js.subjectButtonVisible = shouldShowButton;
        }
        if (t7_readability_js.subjectButtonVisible)
            t7_readability_js.adjustButtonSizeAndPosition();
    },

    process: function() {
        if (t7_readability_js.isMobileSite())
            return;

        try {
            t7_readability_js.init();
        }
        catch (e) {
            t7_readability_js.removeIdForElements(null);
            t7_readability_js.dbg("Unexpected error: " + e);
        }

        function checkZoomLevelDelay() {
            setTimeout(checkZoomLevel, 0);
        }
    },

    onResize: function(event, newScale) {
        setTimeout("t7_readability_js.checkZoomLevel(" + newScale + ")", 0);
    },

    isMobileSite: function() {
        var allMeta = document.getElementsByTagName("meta");
        for (var i=0; i<allMeta.length; i+=1) {
            var meta = allMeta[i];
            if(meta.name === "viewport"
                && meta.content.indexOf("width=device-width") != -1
                && screen.width == document.documentElement.offsetWidth)
                return true;
        }
        return false;
    }
};

// Extracts long-form content from a page and returns and array where the first
// element is the article title, the second element is HTML containing the
// long-form content, and remaining elements are URLs for images referenced by
// that HTML. Each <img> tag in the HTML has an id field set to k - 2, which
// corresponds to a URL listed at index k in the array returned.
(function () {
    t7_readability_js.process();
    var result = new Array(4);
    result[0] = t7_readability_js.getArticleTitle();
    result[1] = t7_readability_js.getDistilledArticleHTML();
    result[2] = JSON.stringify(t7_readability_js.getButtonRect());
    result[3] = t7_readability_js.getNextPageLink();
    return result;
}())

